from typing import Optional

from openstack.exceptions import ResourceNotFound

from interface.network.Router import Router
from interface.network.RouterManager import RouterManager


class OpenstackRouterManager(RouterManager):

    def __init__(self, openstack) -> None:
        super().__init__()

        self.openstack = openstack

    @staticmethod
    def os_router_to_rc_router(os_router) -> Router:
        return Router(
            os_router.id,
            os_router.name,
            os_router.status,
            os_router.external_gateway_info,
            os_router.created_at,
            os_router.updated_at
        )

    def create(self) -> Router:
        os_router = self.openstack.network.create_router()

        return self.os_router_to_rc_router(os_router)

    def get(self, id) -> Optional[Router]:
        try:
            os_router = self.openstack.network.get_router(id)
        except ResourceNotFound:
            return None

        return self.os_router_to_rc_router(os_router)

    def update(self, id, params) -> Optional[Router]:
        try:
            os_router = self.openstack.network.update_router(id, **params)
        except ResourceNotFound:
            return None

        return self.os_router_to_rc_router(os_router)

    def add_interface(self, id, subnet_id):
        try:
            os_router = self.openstack.network.add_interface_to_router(id, subnet_id) # TODO: can't get the id of the router from dict
        except ResourceNotFound:
            return None

        return self.os_router_to_rc_router(os_router)

    def add_gateway(self, id, network_id):
        #try:
        os_router = self.openstack.network.add_gateway_to_router(id, external_gateway_info=network_id)
        #except ResourceNotFound:
        #    return None

        print(os_router)
        return self.os_router_to_rc_router(os_router)

    def delete(self, id) -> bool:
        try:
            self.openstack.network.delete_router(id)
        except ResourceNotFound:
            return False

        return True

    def list(self) -> []:
        routers = []

        for os_router in self.openstack.network.routers():
            routers.append(self.os_router_to_rc_router(os_router))

        return routers
